@@ -363,7 +363,11 @@ int tube_read_msg(struct tube* tube, uint8_t** buf, uint32_t* len,
 		}
 		d += r;
 	}
-	log_assert(*len < 65536*2);
+	if (*len >= 65536*2) {
+		log_err("tube msg length %u is too big", (unsigned)*len);
+		(void)fd_set_nonblock(fd);
+		return 0;
+	}
 	*buf = (uint8_t*)malloc(*len);
 	if(!*buf) {
 		log_err("tube read out of memory");
@@ -420,6 +424,28 @@ int tube_wait(struct tube* tube)
 	return pollit(tube->sr, NULL);
 }
 
+int tube_wait_timeout(struct tube* tube, int msec)
+{
+	struct timeval t;
+	int fd = tube->sr;
+	fd_set r;
+	t.tv_sec = msec/1000;
+	t.tv_usec = (msec%1000)*1000;
+#ifndef S_SPLINT_S
+	FD_ZERO(&r);
+	FD_SET(FD_SET_T fd, &r);
+#endif
+	while(1) {
+		if(select(fd+1, &r, NULL, NULL, &t) == -1) {
+			if(errno == EAGAIN || errno == EINTR)
+				continue;
+			return -1;
+		}
+		break;
+	}
+	return (int)(FD_ISSET(fd, &r));
+}
+
 int tube_read_fd(struct tube* tube)
 {
 	return tube->sr;
@@ -454,8 +480,9 @@ int tube_setup_bg_write(struct tube* tube, struct comm_base* base)
 
 int tube_queue_item(struct tube* tube, uint8_t* msg, size_t len)
 {
-	struct tube_res_list* item = 
-		(struct tube_res_list*)malloc(sizeof(*item));
+	struct tube_res_list* item;
+	if(!tube || !tube->res_com) return 0;
+	item = (struct tube_res_list*)malloc(sizeof(*item));
 	if(!item) {
 		free(msg);
 		log_err("out of memory for async answer");
@@ -644,6 +671,26 @@ int tube_wait(struct tube* tube)
 	return 1;
 }
 
+int tube_wait_timeout(struct tube* tube, int msec)
+{
+	/* block on eventhandle */
+	DWORD res = WSAWaitForMultipleEvents(
+		1 /* one event in array */,
+		&tube->event /* the event to wait for, our pipe signal */,
+		0 /* wait for all events is false */,
+		msec /* wait for timeout */,
+		0 /* we are not alertable for IO completion routines */
+		);
+	if(res == WSA_WAIT_TIMEOUT) {
+		return 0;
+	}
+	if(res == WSA_WAIT_IO_COMPLETION) {
+		/* a bit unexpected, since we were not alertable */
+		return -1;
+	}
+	return 1;
+}
+
 int tube_read_fd(struct tube* ATTR_UNUSED(tube))
 {
 	/* nothing sensible on Windows */
@@ -687,8 +734,9 @@ int tube_setup_bg_write(struct tube* ATTR_UNUSED(tube),
 
 int tube_queue_item(struct tube* tube, uint8_t* msg, size_t len)
 {
-	struct tube_res_list* item = 
-		(struct tube_res_list*)malloc(sizeof(*item));
+	struct tube_res_list* item;
+	if(!tube) return 0;
+	item = (struct tube_res_list*)malloc(sizeof(*item));
 	verbose(VERB_ALGO, "tube queue_item len %d", (int)len);
 	if(!item) {
 		free(msg);
